@@ -8,11 +8,20 @@ the connecting client, rather that the connecting proxy.
 
 https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers#Proxies
 """
-from typing import List
+
+from __future__ import annotations
+
+from typing import Union, cast
+
+from uvicorn._types import ASGI3Application, ASGIReceiveCallable, ASGISendCallable, HTTPScope, Scope, WebSocketScope
 
 
 class ProxyHeadersMiddleware:
-    def __init__(self, app, trusted_hosts="127.0.0.1"):
+    def __init__(
+        self,
+        app: ASGI3Application,
+        trusted_hosts: list[str] | str = "127.0.0.1",
+    ) -> None:
         self.app = app
         if isinstance(trusted_hosts, str):
             self.trusted_hosts = {item.strip() for item in trusted_hosts.split(",")}
@@ -20,9 +29,7 @@ class ProxyHeadersMiddleware:
             self.trusted_hosts = set(trusted_hosts)
         self.always_trust = "*" in self.trusted_hosts
 
-    def get_trusted_client_host(
-        self, x_forwarded_for_hosts
-    ):  # type: (List[str]) -> str
+    def get_trusted_client_host(self, x_forwarded_for_hosts: list[str]) -> str | None:
         if self.always_trust:
             return x_forwarded_for_hosts[0]
 
@@ -30,9 +37,12 @@ class ProxyHeadersMiddleware:
             if host not in self.trusted_hosts:
                 return host
 
-    async def __call__(self, scope, receive, send):
+        return None  # pragma: full coverage
+
+    async def __call__(self, scope: Scope, receive: ASGIReceiveCallable, send: ASGISendCallable) -> None:
         if scope["type"] in ("http", "websocket"):
-            client_addr = scope.get("client")
+            scope = cast(Union["HTTPScope", "WebSocketScope"], scope)
+            client_addr: tuple[str, int] | None = scope.get("client")
             client_host = client_addr[0] if client_addr else None
 
             if self.always_trust or client_host in self.trusted_hosts:
@@ -41,19 +51,20 @@ class ProxyHeadersMiddleware:
                 if b"x-forwarded-proto" in headers:
                     # Determine if the incoming request was http or https based on
                     # the X-Forwarded-Proto header.
-                    x_forwarded_proto = headers[b"x-forwarded-proto"].decode("latin1")
-                    scope["scheme"] = x_forwarded_proto.strip()
+                    x_forwarded_proto = headers[b"x-forwarded-proto"].decode("latin1").strip()
+                    if scope["type"] == "websocket":
+                        scope["scheme"] = x_forwarded_proto.replace("http", "ws")
+                    else:
+                        scope["scheme"] = x_forwarded_proto
 
                 if b"x-forwarded-for" in headers:
                     # Determine the client address from the last trusted IP in the
                     # X-Forwarded-For header. We've lost the connecting client's port
                     # information by now, so only include the host.
                     x_forwarded_for = headers[b"x-forwarded-for"].decode("latin1")
-                    x_forwarded_for_hosts = [
-                        item.strip() for item in x_forwarded_for.split(",")
-                    ]
+                    x_forwarded_for_hosts = [item.strip() for item in x_forwarded_for.split(",")]
                     host = self.get_trusted_client_host(x_forwarded_for_hosts)
                     port = 0
-                    scope["client"] = (host, port)
+                    scope["client"] = (host, port)  # type: ignore[arg-type]
 
         return await self.app(scope, receive, send)
